from .base import Repository
from ..configuration import get_snippets_home_path
from ..snippet import parse_snippet
import logging
import os


class LocalRepository(Repository):
    def create_snippet(self, snippet_id):
        raise NotImplementedError()

    def __init__(self):
        self._root = get_snippets_home_path()
        logging.debug("root directory: {}".format(self._root))

    def get_snippet_local_path(self, snippet_id):
        local_path = snippet_id
        path = os.path.join(self._root, local_path)
        return path

    def get_snippet(self, snippet_id):
        local_path = snippet_id
        path = os.path.join(self._root, local_path)

        if os.path.isfile(path):
            with open(path, "r") as fp:
                file_content = fp.read()

            return parse_snippet(file_content)
        else:
            logging.error("snippet doesn't exist: {}".format(path))

    def list(self):
        for root, dirs, files in os.walk(self._root):
            dirs[:] = [d for d in dirs if not d.startswith(".")]
            files[:] = [file for file in files if not file.startswith(".")]

            for file in files:
                full_path = os.path.join(root, file)
                relative_path = os.path.relpath(full_path, self._root)
                yield relative_path



from .utilities import highlight, open_with_editor
from .configuration import get_snippets_editor
import logging
import tempfile
import yaml
import os


def list_snippets(repository):
    # list all snippets
    for snippet in repository.list():
        print(snippet)


def show_snippet(repository, snippet):
    # show the snippet
    snippet = repository.get_snippet(snippet)
    language = snippet.metadata["language"]

    print(highlight(language, snippet.snippet))


def snippet_info(repository, snippet):
    snippet = repository.get_snippet(snippet)
    print(snippet.metadata)


def edit_snippet(repository, snippet_id):
    # edit the snippet
    logging.debug("snippet: {}".format(snippet_id))
    editor = get_snippets_editor()

    snippet = repository.get_snippet(snippet_id)
    snippet_path = repository.get_snippet_local_path(snippet_id)

    # copy snippet to temporary file, without the header
    os_descriptor, path = tempfile.mkstemp()
    with os.fdopen(os_descriptor, "w") as fh:
        fh.write(snippet.snippet)

    if not editor:
        logging.error("editor not available.")
    else:
        open_with_editor(editor, path)

    with open(path, "r") as fh:
        new_snippet = fh.read()

    with open(snippet_path, "w") as fh:
        fh.write("---\n")
        fh.write(yaml.dump(snippet.metadata, default_flow_style=False))
        fh.write("---\n")
        fh.write(new_snippet)



"""snip.

Usage:
    snip <snippet>
    snip --info <snippet>
    snip --edit <snippet>
    snip --list
    snip --version|-v

"""
from .commands import edit_snippet, snippet_info, show_snippet, list_snippets
from .repository.local import LocalRepository
from docopt import docopt
import logging


def main():
    logging.basicConfig(level=logging.DEBUG)

    arguments = docopt(__doc__)
    logging.debug("arguments: {}".format(arguments))

    repository = LocalRepository()

    if arguments["--list"]:
        # list all snippets
        list_snippets(repository)
    elif arguments["--info"]:
        # prints some info about the selected snippet
        snippet = arguments["<snippet>"]
        snippet_info(repository, snippet)
    elif arguments["--edit"]:
        snippet = arguments["<snippet>"]
        edit_snippet(repository, snippet)
    else:
        # show the snippet
        snippet = arguments["<snippet>"]
        show_snippet(repository, snippet)


if __name__ == "__main__":
    main()



import pygments
import subprocess
import logging
from pygments.lexers import get_lexer_by_name
from pygments.formatters import TerminalFormatter


def highlight(language, code):
    lexer = get_lexer_by_name(language)
    return pygments.highlight(code, lexer, TerminalFormatter())


def open_with_editor(editor, filepath):
    try:
        subprocess.call([editor, filepath])
    except OSError:
        logging.error('Could not launch ' + editor)



import os
import logging


def get_snippets_home_path():
    snippets_home_path = os.environ.get("SNIPPETS_DIR", "~/.snippets")
    snippets_home_path = os.path.expanduser(
        os.path.expandvars(snippets_home_path))

    if not os.path.isdir(snippets_home_path):
        logging.warning("Snippets home directory doesn't exist!")
        os.mkdir(snippets_home_path)
    else:
        write_permission = os.access(snippets_home_path, os.W_OK)
        read_permission = os.access(snippets_home_path, os.R_OK)

        if not write_permission or not read_permission:
            logging.error("Doesn't seems you have the rights to access this directory, "
                          "something wrong with permissions?")

    return snippets_home_path


def get_snippets_editor():
    return os.environ.get("SNIPPET_EDITOR") or os.environ.get("VISUAL") or os.environ.get("EDITOR")



from ..snippet import parse_snippet
from .base import Repository
import requests
import json
import base64


class GithubApi:
    @staticmethod
    def repository_tree(owner, repository, tree_hash, recursive):
        url = "https://api.github.com/repos/{}/{}/git/trees/{}?recursive={}".format(
            owner, repository, tree_hash, recursive)
        return requests.get(url)

    @staticmethod
    def repository_content(owner, repository, path):
        url = "https://api.github.com/repos/{}/{}/contents/{}".format(
            owner, repository, path)
        return requests.get(url)


class GithubNode:
    @property
    def path(self):
        return self._path

    def __init__(self, sha, path):
        self._sha = sha
        self._path = path


class GithubTree:
    def __init__(self):
        self._children = []

    def add_node(self, node):
        self._children.append(node)


class GithubRepository(Repository):
    @property
    def owner(self):
        return self._owner

    @property
    def repository(self):
        return self._repository

    @property
    def refs(self):
        return self._refs

    def __init__(self, owner, repository, refs="HEAD"):
        self._owner = owner
        self._repository = repository
        self._refs = refs
        self._root = GithubTree()

    def create_snippet(self, snippet_id):
        raise NotImplementedError()

    def get_snippet(self, snippet_id):
        if not self._root:
            raise NotImplementedError()
        else:
            for child in self._root._children:
                if child.path == snippet_id:
                    response = GithubApi.repository_content(
                        self.owner, self.repository, child.path)
                    content_response = json.loads(
                        response.content.decode(response.encoding))
                    content = base64.b64decode(
                        content_response["content"]).decode("utf-8")
                    return parse_snippet(content)

    def list(self):
        self._root = GithubTree()
        return self._recursive_path_explorer(self._root, self.owner, self.repository, self.refs, "1")

    def _recursive_path_explorer(self, root, owner, repository, tree_hash, recursive):
        response = GithubApi.repository_tree(
            owner, repository, tree_hash, recursive)

        tree_response = json.loads(response.content.decode(response.encoding))

        current_tree = tree_response["tree"]
        current_tree_truncated = tree_response["truncated"]

        for child in current_tree:
            child_type = child["type"]
            if child_type == "tree" and current_tree_truncated:
                subtree = GithubTree(child["sha"], child["path"])
                root.add_node(subtree)
                self._recursive_path_explorer(
                    subtree, owner, repository, child["sha"], "1")
            elif child_type == "blob":
                root.add_node(GithubNode(child["sha"], child["path"]))
                yield child["path"]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="snipped",
    version="0.1a0",
    author="Fabrizio Destro",
    author_email="destro.fabrizio@gmail.com",
    description="A small snippets manager.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dexpota/snip",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'snip=snip.main:main',
        ],
    },
    install_requires=[
        "docopt",
        "pyyaml"
        "pygments"
        "requests"],
)



import yaml
import re


class Snippet:
    @property
    def metadata(self):
        return self._metadata

    @property
    def snippet(self):
        return self._snippet

    def __init__(self, metadata: str, snippet: str):
        self._metadata = metadata
        self._snippet = snippet


def parse_snippet(file_content: str) -> Snippet:
    splitter = re.compile(r'^-{3}[\s\S]', re.MULTILINE)
    _, metadata_content, snippet_content = splitter.split(file_content, 2)
    metadata = yaml.load(metadata_content)
    return Snippet(metadata, snippet_content)




class Repository:
    def create_snippet(self, snippet_id):
        raise NotImplementedError()

    def get_snippet(self, snippet_id):
        raise NotImplementedError()

    def list(self):
        raise NotImplementedError()
